-- options stuff-----------------------------------------------------------------------
-- Upvalued Lua API.
-----------------------------------------------------------------------
local _G = getfenv(0)

-- Functions
local pairs = _G.pairs

-- Libraries
local table = _G.table

-----------------------------------------------------------------------
-- AddOn namespace.
-----------------------------------------------------------------------
local ADDON_NAME, private = ...

local LibStub = _G.LibStub

local BTC = LibStub("AceAddon-3.0"):GetAddon(ADDON_NAME, "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(ADDON_NAME)
local LDBIcon = LibStub("LibDBIcon-1.0")

------------------------------------------------------------------------------
-- Constants.
------------------------------------------------------------------------------
local ADDON_NAME_SHORT = private.ADDON_NAME_SHORT
local ADDON_AUTHOR = "James D. Callahan III"
local ADDON_BUILD_DATE = "20121029234358"

------------------------------------------------------------------------------
-- Variables.
------------------------------------------------------------------------------
local db

local display_options
local options
local general_options
local text_options

local spell_registry = {}

local function DisplayOptions()
	if display_options then
		return display_options
	end
	display_options = {
		order = 1,
		type = "group",
		name = _G.DISPLAY_OPTIONS,
		desc = L["DISPLAY_OPTIONS_DESC"],
		args = {
			header1 = {
				order = 1,
				type = "header",
				name = _G.DISPLAY_OPTIONS,
			},
			desc = {
				order = 1.1,
				type = "description",
				name = L["DISPLAY_OPTIONS_DESC"],
			},
			tooltipscale = {
				order = 1.2,
				type = "range",
				name = L["Tooltip Scale"],
				desc = L["TOOLTIP_SCALE_DESC"],
				min = 0.5,
				max = 1.5,
				step = .05,
				bigStep = .05,
				get = function()
					return db.options.tipscale
				end,
				set = function(info, v)
					db.options.tipscale = v
				end,
			},
			showprimary = {
				order = 1.3,
				type = "toggle",
				name = L["Show Primary"],
				desc = L["SHOW_PRIMARY_DESC"],
				get = function()
					return db.options.showPrimary
				end,
				set = function()
					db.options.showPrimary = not db.options.showPrimary
				end,
			},
			showsecondary = {
				order = 1.4,
				type = "toggle",
				name = L["Show Secondary"],
				desc = L["SHOW_SECONDARY_DESC"],
				get = function()
					return db.options.showSecondary
				end,
				set = function()
					db.options.showSecondary = not db.options.showSecondary
				end,
			},
			hidereminders = {
				order = 1.5,
				type = "toggle",
				name = L["Hide Reminders"],
				desc = L["HIDE_REMINDERS_DESC"],
				get = function()
					return db.options.hideReminders
				end,
				set = function()
					db.options.hideReminders = not db.options.hideReminders
				end,
			},
			--[===[@debug@
			debug = {
				order = 1.6,
				type = "toggle",
				name = L["Debug"],
				desc = L["DEBUG_DESC"],
				get = function()
					return db.options.debug
				end,
				set = function()
					db.options.debug = not db.options.debug
				end,
			},
			--@end-debug@]===]
			header2 = {
				order = 2,
				type = "header",
				name = L["Minimap Icon Options"],
			},
			desc2 = {
				order = 2.1,
				type = "description",
				name = L["MINIMAP_OPTIONS_DESC"],
			},
			minimap_icon = {
				order = 2.2,
				type = "toggle",
				width = "full",
				name = L["Show Minimap Icon"],
				desc = L["Draws the icon on the minimap."],
				get = function()
					return not db.options.minimap_icon.hide
				end,
				set = function(info, value)
					db.options.minimap_icon.hide = not value
					LDBIcon[value and "Show" or "Hide"](LDBIcon, ADDON_NAME_SHORT)
				end,
			},
			header3 = {
				order = 3,
				type = "header",
				name = L["Help and Usage Notes"],
			},
			help1 = {
				order = 3.1,
				type = "description",
				name = "- " .. L["HELP_STRING_0"],
			},
			help2 = {
				order = 3.2,
				type = "description",
				name = "- " .. L["HELP_STRING_1"],
			},
			help3 = {
				order = 3.3,
				type = "description",
				name = "- " .. L["HELP_STRING_2"],
			},
			help4 = {
				order = 3.4,
				type = "description",
				name = "- " .. L["HELP_STRING_3"],
			},
			help5 = {
				order = 3.5,
				type = "description",
				name = "- " .. L["HELP_STRING_4"],
			},
		},
	}
	return display_options
end

local function GeneralOptions()
	if general_options then
		return general_options
	end
	general_options = {
		order = 3,
		type = "group",
		name = L["General Information"],
		desc = L["GENERAL_INFO_DESC"],
		args = {
			header1 = {
				order = 1,
				type = "header",
				name = L["General Information"],
			},
			desc = {
				order = 1.1,
				type = "description",
				name = L["GENERAL_INFO_DESC"],
			},
			author = {
				order = 1.2,
				type = "description",
				name = L["Author : "] .. ADDON_AUTHOR .. "\n",
			},
			version = {
				order = 1.3,
				type = "description",
				name = L["Version : "] .. private.ADDON_VERSION .. "\n",
			},
			builddate = {
				order = 1.4,
				type = "description",
				name = L["Build Date : "] .. ADDON_BUILD_DATE .. "\n",
			},
		},
	}
	return general_options
end

local function TextOptions()
	if text_options then
		return text_options
	end
	text_options = {
		order = 2,
		type = "group",
		name = L["Text Options"],
		desc = L["TEXT_OPTIONS_DESC"],
		args = {
			header1 = {
				order = 1,
				type = "header",
				name = L["Text Options"],
			},
			desc = {
				order = 1.2,
				type = "description",
				name = L["TEXT_OPTIONS_DESC"],
			},
			showtext = {
				order = 1.3,
				type = "toggle",
				name = L["Show Text"],
				desc = L["SHOW_TEXT_DESC"],
				get = function()
					return db.options.showText
				end,
				set = function()
					db.options.showText = not db.options.showText
					private.UpdateDataText()
				end,
			},
			showcounters = {
				order = 1.4,
				type = "toggle",
				name = L["Show Counters"],
				desc = L["SHOW_COUNTERS_DESC"],
				get = function()
					return db.options.showCounters
				end,
				set = function()
					db.options.showCounters = not db.options.showCounters
					private.UpdateDataText()
				end,
				disabled = function()
					return not db.options.showText
				end,
			},
			textcooldown = {
				order = 1.5,
				type = "select",
				name = L["Select Cooldown to Display"],
				desc = L["SHOW_TEXTCD_DESC"],
				get = function()
					return db.options.textCooldown
				end,
				set = function(info, value)
					db.options.textCooldown = value
					private.UpdateDataText()
				end,
				values = function()
					local t = {}

					for character_name, data in pairs(BTC.db.factionrealm) do
						table.wipe(spell_registry)

						for spell_id, cooldown_data in pairs(data.cooldowns) do
							local display_name = private.CooldownDisplayName(spell_id)

							if not cooldown_data.hide and not spell_registry[display_name] then
								t[character_name .. ":" .. spell_id] = _G.SUBTITLE_FORMAT:format(character_name, display_name)
								spell_registry[display_name] = true
							end
						end
					end
					return t
				end,
				disabled = function()
					return not db.options.showText or db.options.showCounters
				end,
			},
		},
	}
	return text_options
end

local function Options()
	if options then
		return options
	end

	options = {
		name = ADDON_NAME_SHORT,
		type = "group",
		childGroups = "tab",
		args = {},
	}

	options.args.general_options = GeneralOptions()
	options.args.text_options = TextOptions()
	options.args.display_options = DisplayOptions()
	return options
end

function BTC:SetupOptions()
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(ADDON_NAME_SHORT, Options)
	self.options_frame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(ADDON_NAME_SHORT)
	db = self.db.factionrealm[private.PLAYER_NAME]
end
